"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Worker = void 0;
const constant = require("../type/constant");
const axios_1 = require("axios");
const unique_string_generator_1 = require("./unique-string-generator");
class Worker {
    constructor(mainnet) {
        this.sleep = (milliseconds) => {
            return new Promise((resolve) => setTimeout(resolve, milliseconds));
        };
        this.uniqueStringGenerator = new unique_string_generator_1.Generator();
        if (mainnet) {
            this.requestUrl = constant.REQUEST_MAINNET;
            this.pollUrl = constant.POLL_MAINNET;
            this.cancelConnectionUrl = constant.CANCEL_CONNECT_MAINNET;
        }
        else {
            this.requestUrl = constant.REQUEST_TESTNET;
            this.pollUrl = constant.POLL_TESTNET;
            this.cancelConnectionUrl = constant.CANCEL_CONNECT_TESTNET;
        }
    }
    /**
     * will return a empty array if there are no wallet objects.
     * @returns {wallet[]} an array of wallet object consisting of ```wallet_name``` and ```public_address```.
     */
    getWallets() {
        if (this.wallets === null)
            return [];
        else
            return this.wallets;
    }
    /**
     * @return {string} unique string that can be used to generate qr
     * @param iconUrl your website icon url
     * @param websiteUrl your website url
     * @param websiteName your website name
     */
    request(iconUrl, websiteUrl, websiteName) {
        return __awaiter(this, void 0, void 0, function* () {
            let request = yield axios_1.default.post(this.requestUrl);
            let response = request.data;
            this.setUuid(response.data.uuid);
            const uniqueString = this.generateUniqueString(iconUrl, websiteUrl, websiteName, response.data.uuid);
            return uniqueString;
        });
    }
    /**
     * will poll the server to connect.
     * @param timeout time the generated unique string will be valid in ```miliseconds```(default 30 seconds)
     * @param pollRate interval for each poll in miliseconds (default to 1 second)
     */
    connect(timeout = constant.SECOND * 30, pollRate = 1000) {
        return __awaiter(this, void 0, void 0, function* () {
            if (timeout === "never") {
                const check = yield this.checkConnection();
                this.setConnectionStatus(check.message);
                while (this.isPending()) {
                    const check = yield this.checkConnection();
                    this.setConnectionStatus(check.message);
                    this.setWallets(check.data);
                    yield this.sleep(pollRate);
                }
                return this.response;
            }
            else {
                const expiry = this.getExpiry(timeout);
                while (expiry.expiry >= Date.now()) {
                    const check = yield this.checkConnection();
                    this.setConnectionStatus(check.message);
                    this.setWallets(check.data);
                    if (!this.isPending())
                        break;
                    yield this.sleep(pollRate);
                }
                // cancel the connection if the connection is still pending after expiring
                if (this.isPending()) {
                    return yield this.cancelConnection();
                }
                return this.response;
            }
        });
    }
    setUuid(uuid) {
        this.uuid = uuid;
    }
    cancelConnection() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const request = yield axios_1.default.post(this.cancelConnectionUrl, this.getBodyData());
                const response = request.data;
                return response;
            }
            catch (error) {
                console.error(error);
                throw new Error(error);
            }
        });
    }
    isPending() {
        switch (this.connectionStatus) {
            case constant.PENDING:
                return true;
            default:
                return false;
        }
    }
    setConnectionStatus(status) {
        this.connectionStatus = status;
    }
    setWallets(wallets) {
        this.wallets = wallets;
    }
    setResponse(response) {
        this.response = response;
    }
    checkConnection() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const request = yield axios_1.default.post(this.pollUrl, this.getBodyData());
                const response = request.data;
                this.setResponse(response);
                return response;
            }
            catch (error) {
                console.error(error);
                throw new Error(error);
            }
        });
    }
    getBodyData() {
        const data = {
            uuid: this.uuid,
        };
        let body = data;
        return body;
    }
    getExpiry(timeout) {
        const timestammp = Date.now();
        return {
            created: timestammp,
            expiry: timestammp + timeout,
        };
    }
    generateUniqueString(iconUrl, websiteUrl, websiteName, uuid) {
        const uniqueString = this.uniqueStringGenerator.generate(iconUrl, websiteUrl, websiteName, uuid);
        return uniqueString;
    }
}
exports.Worker = Worker;
